@@ -11,6 +11,7 @@
 import __future__
 import sys
 import numpy
+import threading
 
 from numexpr import interpreter, expressions, use_vml, is_cpu_amd_intel
 from numexpr.utils import CacheDict
@@ -34,6 +35,36 @@ type_to_kind = expressions.type_to_kind
 kind_to_type = expressions.kind_to_type
 default_type = kind_to_type[expressions.default_kind]
 
+# VML functions that are implemented in numexpr
+vml_functions = [
+    "div",  # interp_body.cpp
+    "inv",  # interp_body.cpp
+    "pow",  # interp_body.cpp
+    # Keep the rest of this list in sync with the ones listed in functions.hpp
+    "sqrt",
+    "sin",
+    "cos",
+    "tan",
+    "arcsin",
+    "arccos",
+    "arctan",
+    "sinh",
+    "cosh",
+    "tanh",
+    "arcsinh",
+    "arccosh",
+    "arctanh",
+    "log",
+    "log1p",
+    "log10",
+    "exp",
+    "expm1",
+    "abs",
+    "conj",
+    "arctan2",
+    "fmod",
+    ]
+
 # Final addtions for Python 3 (mainly for PyTables needs)
 if sys.version_info[0] > 2:
     typecode_to_kind['s'] = 'str'
@@ -261,7 +292,8 @@ def stringToExpression(s, types, context):
 
 
 def isReduction(ast):
-    return ast.value.startswith(b'sum_') or ast.value.startswith(b'prod_')
+    prefixes = (b'sum_', b'prod_', b'min_', b'max_')
+    return any(ast.value.startswith(p) for p in prefixes)
 
 
 def getInputOrder(ast, input_order=None):
@@ -663,15 +695,7 @@ def getExprNames(text, context):
         ex_uses_vml = False
     else:
         for node in ast.postorderWalk():
-            if node.astType == 'op' \
-                    and node.value in ['sin', 'cos', 'exp', 'log',
-                                       'expm1', 'log1p',
-                                       'pow', 'div',
-                                       'sqrt', 'inv',
-                                       'sinh', 'cosh', 'tanh',
-                                       'arcsin', 'arccos', 'arctan',
-                                       'arccosh', 'arcsinh', 'arctanh',
-                                       'arctan2', 'abs']:
+            if node.astType == 'op' and node.value in vml_functions:
                 ex_uses_vml = True
                 break
         else:
@@ -684,6 +708,7 @@ def getExprNames(text, context):
 _names_cache = CacheDict(256)
 _numexpr_cache = CacheDict(256)
 
+evaluate_lock = threading.Lock()
 
 def evaluate(ex, local_dict=None, global_dict=None,
              out=None, order='K', casting='safe', **kwargs):
@@ -729,39 +754,40 @@ def evaluate(ex, local_dict=None, global_dict=None,
             like float64 to float32, are allowed.
           * 'unsafe' means any data conversions may be done.
     """
-    if not isinstance(ex, (str, unicode)):
-        raise ValueError("must specify expression as a string")
-    # Get the names for this expression
-    context = getContext(kwargs, frame_depth=1)
-    expr_key = (ex, tuple(sorted(context.items())))
-    if expr_key not in _names_cache:
-        _names_cache[expr_key] = getExprNames(ex, context)
-    names, ex_uses_vml = _names_cache[expr_key]
-    # Get the arguments based on the names.
-    call_frame = sys._getframe(1)
-    if local_dict is None:
-        local_dict = call_frame.f_locals
-    if global_dict is None:
-        global_dict = call_frame.f_globals
+    with evaluate_lock:
+        if not isinstance(ex, (str, unicode)):
+            raise ValueError("must specify expression as a string")
+        # Get the names for this expression
+        context = getContext(kwargs, frame_depth=1)
+        expr_key = (ex, tuple(sorted(context.items())))
+        if expr_key not in _names_cache:
+            _names_cache[expr_key] = getExprNames(ex, context)
+        names, ex_uses_vml = _names_cache[expr_key]
+        # Get the arguments based on the names.
+        call_frame = sys._getframe(1)
+        if local_dict is None:
+            local_dict = call_frame.f_locals
+        if global_dict is None:
+            global_dict = call_frame.f_globals
 
-    arguments = []
-    for name in names:
+        arguments = []
+        for name in names:
+            try:
+                a = local_dict[name]
+            except KeyError:
+                a = global_dict[name]
+            arguments.append(numpy.asarray(a))
+
+        # Create a signature
+        signature = [(name, getType(arg)) for (name, arg) in zip(names, arguments)]
+
+        # Look up numexpr if possible.
+        numexpr_key = expr_key + (tuple(signature),)
         try:
-            a = local_dict[name]
+            compiled_ex = _numexpr_cache[numexpr_key]
         except KeyError:
-            a = global_dict[name]
-        arguments.append(numpy.asarray(a))
-
-    # Create a signature
-    signature = [(name, getType(arg)) for (name, arg) in zip(names, arguments)]
-
-    # Look up numexpr if possible.
-    numexpr_key = expr_key + (tuple(signature),)
-    try:
-        compiled_ex = _numexpr_cache[numexpr_key]
-    except KeyError:
-        compiled_ex = _numexpr_cache[numexpr_key] = \
-            NumExpr(ex, signature, **context)
-    kwargs = {'out': out, 'order': order, 'casting': casting,
-              'ex_uses_vml': ex_uses_vml}
-    return compiled_ex(*arguments, **kwargs)
+            compiled_ex = _numexpr_cache[numexpr_key] = \
+                NumExpr(ex, signature, **context)
+        kwargs = {'out': out, 'order': order, 'casting': casting,
+                  'ex_uses_vml': ex_uses_vml}
+        return compiled_ex(*arguments, **kwargs)
